use bevy::ecs::schedule::ShouldRun;
use bevy::{prelude::*, window::PresentMode, winit::WinitSettings};
use bevy_egui::{egui, EguiContext, EguiPlugin};
use bevy_prototype_lyon::prelude::*;

use shape_maker::*;

fn main() {
    let mut app = App::new();

    app
        .insert_resource(ClearColor(Color::rgb(0.1, 0.1, 0.12)))
        .insert_resource(Msaa { samples: 4 })
        // Optimal power saving and present mode settings for desktop apps.
        .insert_resource(WinitSettings::desktop_app())
        .insert_resource(WindowDescriptor {
            present_mode: PresentMode::Mailbox,
            title: "Shape Maker".to_string(),
            ..Default::default()
        })
        .insert_resource(ButtonsColors::default())
        .insert_resource(UiState::default())
        .insert_resource(PointSize(5.0))
    ;

    app
        .add_plugins(DefaultPlugins)
        .add_plugin(EguiPlugin)
        .add_plugin(ShapePlugin)
    ;
    app
        .add_startup_system(configure_visuals)
        .add_startup_system(basic_setup_sys)
        .add_system(create_sys.with_run_criteria(|state: Res<UiState>, mut ec: ResMut<EguiContext>| 
            if !ec.ctx_mut().is_pointer_over_area() && state.current_action == Action::Create { ShouldRun::Yes } else { ShouldRun::No }
        ))
        .add_system(modify_sys.with_run_criteria(|state: Res<UiState>, mut ec: ResMut<EguiContext>| 
            if !ec.ctx_mut().is_pointer_over_area() && state.current_action == Action::Modify { ShouldRun::Yes } else { ShouldRun::No }
        ))
        .add_system(rotate_sys.with_run_criteria(|state: Res<UiState>, mut ec: ResMut<EguiContext>| 
            if !ec.ctx_mut().is_pointer_over_area() && state.current_action == Action::Rotate { ShouldRun::Yes } else { ShouldRun::No }
        ))
        .add_system(action_bar_sys)
        .add_system(shape_tree_sys)
    ;
    
    app.run();
}

fn basic_setup_sys(
    mut coms: Commands,
) {
    coms.spawn_bundle(OrthographicCameraBundle::new_2d())
        .insert(MainCamera);
}

fn configure_visuals(mut egui_ctx: ResMut<EguiContext>) {
    egui_ctx.ctx_mut().set_visuals(egui::Visuals {
        window_rounding: 0.0.into(),
        ..Default::default()
    });
}